@@ -1,19 +1,9 @@
 # © 2016 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 import logging
 import multiprocessing
@@ -25,29 +15,56 @@ from collections.abc import Iterable
 from contextlib import suppress
 from functools import wraps
 from io import StringIO
-from math import isclose
+from math import isclose, isfinite
 from pathlib import Path
-from typing import Any, Sequence, TypeVar
+from typing import Any, Sequence
 
+import img2pdf
 import pikepdf
+from packaging.version import Version
 
 log = logging.getLogger(__name__)
 
+if Version(img2pdf.__version__) < Version('0.4.0'):
+    IMG2PDF_KWARGS = dict(without_pdfw=True)
+elif Version(img2pdf.__version__) < Version('0.4.3'):
+    IMG2PDF_KWARGS = dict(engine=img2pdf.Engine.pikepdf)
+else:
+    IMG2PDF_KWARGS = dict(
+        engine=img2pdf.Engine.pikepdf, rotation=img2pdf.Rotation.ifvalid
+    )
+
 
 class Resolution(namedtuple('Resolution', ('x', 'y'))):
-    """The number of pixels per inch in each 2D direction."""
+    """The number of pixels per inch in each 2D direction.
+
+    Resolution objects are considered "equal" for == purposes if they are
+    equal to a reasonable tolerance.
+    """
 
     __slots__ = ()
 
+    # rel_tol after converting from dpi to pixels per meter and saving
+    # as integer with rounding, as many file formats
+    CONVERSION_ERROR = 0.002
+
     def round(self, ndigits: int):
         return Resolution(round(self.x, ndigits), round(self.y, ndigits))
 
     def to_int(self):
         return Resolution(int(round(self.x)), int(round(self.y)))
 
+    @classmethod
+    def _isclose(cls, a, b):
+        return isclose(a, b, rel_tol=cls.CONVERSION_ERROR)
+
     @property
     def is_square(self) -> bool:
-        return isclose(self.x, self.y, rel_tol=1e-3)
+        return self._isclose(self.x, self.y)
+
+    @property
+    def is_finite(self) -> bool:
+        return isfinite(self.x) and isfinite(self.y)
 
     def take_max(self, vals, yvals=None):
         if yvals is not None:
@@ -64,13 +81,29 @@ class Resolution(namedtuple('Resolution', ('x', 'y'))):
     def __str__(self):
         return f"{self.x:f}x{self.y:f}"
 
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return f"Resolution({self.x}x{self.y} dpi)"
 
+    def __eq__(self, other):
+        if isinstance(other, tuple) and len(other) == 2:
+            other = Resolution(*other)
+        if not isinstance(other, Resolution):
+            return NotImplemented
+        return self._isclose(self.x, other.x) and self._isclose(self.y, other.y)
+
+
+class NeverRaise(Exception):
+    """An exception that is never raised"""
+
 
 def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike):
-    """
-    Helper function: relinks soft symbolic link if necessary
+    """Create a symbolic link at ``soft_link_name``, which references ``input_file``.
+
+    Think of this as copying ``input_file`` to ``soft_link_name`` with less overhead.
+
+    Use symlinks safely. Self-linking loops are prevented. On Windows, file copy is
+    used since symlinks may require administrator privileges. An existing link at the
+    destination is removed.
     """
     input_file = os.fspath(input_file)
     soft_link_name = os.fspath(soft_link_name)
@@ -78,8 +111,8 @@ def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike):
     # Guard against soft linking to oneself
     if input_file == soft_link_name:
         log.warning(
-            "No symbolic link made. You are using "
-            "the original data directory as the working directory."
+            "No symbolic link created. You are using  the original data directory "
+            "as the working directory."
         )
         return
 
@@ -88,10 +121,7 @@ def safe_symlink(input_file: os.PathLike, soft_link_name: os.PathLike):
         # do not delete or overwrite real (non-soft link) file
         if not os.path.islink(soft_link_name):
             raise FileExistsError(f"{soft_link_name} exists and is not a link")
-        try:
-            os.unlink(soft_link_name)
-        except OSError:
-            log.debug("Can't unlink %s", soft_link_name)
+        os.unlink(soft_link_name)
 
     if not os.path.exists(input_file):
         raise FileNotFoundError(f"trying to create a broken symlink to {input_file}")
@@ -120,7 +150,7 @@ def is_iterable_notstr(thing: Any) -> bool:
 
 
 def monotonic(L: Sequence) -> bool:
-    """Does list increase monotonically?"""
+    """Does this sequence increase monotonically?"""
     return all(b > a for a, b in zip(L, L[1:]))
 
 
@@ -170,7 +200,7 @@ def is_file_writable(test_file: os.PathLike) -> bool:
                 with suppress(OSError):
                     p.unlink()
             return True
-    except (EnvironmentError, RuntimeError) as e:
+    except (OSError, RuntimeError) as e:
         log.debug(e)
         log.error(str(e))
         return False
@@ -179,65 +209,82 @@ def is_file_writable(test_file: os.PathLike) -> bool:
 def check_pdf(input_file: Path) -> bool:
     """Check if a PDF complies with the PDF specification.
 
-    Checks for proper formatting and proper linearization.
+    Checks for proper formatting and proper linearization. Uses pikepdf (which in
+    turn, uses QPDF) to perform the checks.
     """
-    pdf = None
     try:
         pdf = pikepdf.open(input_file)
     except pikepdf.PdfError as e:
         log.error(e)
         return False
     else:
-        messages = pdf.check()
-        for msg in messages:
-            if 'error' in msg.lower():
-                log.error(msg)
+        with pdf:
+            messages = pdf.check()
+            success = True
+            for msg in messages:
+                if 'error' in msg.lower():
+                    log.error(msg)
+                    success = False
+                elif (
+                    "/DecodeParms: operation for dictionary attempted on object "
+                    "of type null" in msg
+                ):
+                    pass  # Ignore/spurious warning
+                else:
+                    log.warning(msg)
+                    success = False
+
+            sio = StringIO()
+            linearize_msgs = ''
+            try:
+                # If linearization is missing entirely, we do not complain. We do
+                # complain if linearization is present but incorrect.
+                pdf.check_linearization(sio)
+            except (RuntimeError, pikepdf.ForeignObjectError):
+                pass
             else:
-                log.warning(msg)
+                linearize_msgs = sio.getvalue()
+                if linearize_msgs:
+                    log.warning(linearize_msgs)
 
-        sio = StringIO()
-        linearize = None
-        try:
-            pdf.check_linearization(sio)
-        except RuntimeError:
-            pass
-        else:
-            linearize = sio.getvalue()
-            if linearize:
-                log.warning(linearize)
-
-        if not messages and not linearize:
-            return True
-        return False
-    finally:
-        if pdf:
-            pdf.close()
+            if success and not linearize_msgs:
+                return True
+            return False
 
 
-T = TypeVar('T')
-
-
-def clamp(n: T, smallest: T, largest: T) -> T:
-    """Clamps the value of n to between smallest and largest."""
+def clamp(n, smallest, largest):  # mypy doesn't understand types for this
+    """Clamps the value of ``n`` to between ``smallest`` and ``largest``."""
     return max(smallest, min(n, largest))
 
 
+def remove_all_log_handlers(logger):
+    "Remove all log handlers, usually used in a child process."
+    for handler in logger.handlers[:]:
+        logger.removeHandler(handler)
+        handler.close()  # To ensure handlers with opened resources are released
+
+
 def pikepdf_enable_mmap():
-    try:
-        if pikepdf._qpdf.set_access_default_mmap(True):
-            log.debug("pikepdf mmap enabled")
-    except AttributeError:
-        log.debug("pikepdf mmap not available")
+    # try:
+    #     if pikepdf._qpdf.set_access_default_mmap(True):
+    #         log.debug("pikepdf mmap enabled")
+    # except AttributeError:
+    #     log.debug("pikepdf mmap not available")
+    # We found a race condition probably related to pybind issue #2252 that can
+    # cause a crash. For now, disable pikepdf mmap to be on the safe side.
+    # Fix is not in pybind11 2.6.0
+    # log.debug("pikepdf mmap disabled")
+    return
 
 
 def deprecated(func):
-    """Warn that function is deprecated"""
+    """Warn that function is deprecated."""
 
     @wraps(func)
     def new_func(*args, **kwargs):
         warnings.simplefilter('always', DeprecationWarning)  # turn off filter
         warnings.warn(
-            "Call to deprecated function {}.".format(func.__name__),
+            f"Call to deprecated function {func.__name__}.",
             category=DeprecationWarning,
             stacklevel=2,
         )
