@@ -1,66 +1,39 @@
 # © 2017 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 import os
 import platform
 import sys
 from pathlib import Path
-from subprocess import PIPE, run
+from subprocess import PIPE, CompletedProcess, run
+from typing import List
 
 import pytest
 
 from ocrmypdf import api, pdfinfo
 from ocrmypdf._exec import unpaper
 from ocrmypdf._plugin_manager import get_parser_options_plugins
-
-pytest_plugins = ['helpers_namespace']
+from ocrmypdf.exceptions import ExitCode
 
 
-# pylint: disable=E1101
-# pytest.helpers is dynamic so it confuses pylint
-
-if sys.version_info < (3, 5):
-    print("Requires Python 3.5+")
-    sys.exit(1)
-
-
-@pytest.helpers.register
 def is_linux():
     return platform.system() == 'Linux'
 
 
-@pytest.helpers.register
 def is_macos():
     return platform.system() == 'Darwin'
 
 
-@pytest.helpers.register
 def running_in_docker():
     # Docker creates a file named /.dockerenv (newer versions) or
     # /.dockerinit (older) -- this is undocumented, not an offical test
     return Path('/.dockerenv').exists() or Path('/.dockerinit').exists()
 
 
-@pytest.helpers.register
-def running_in_travis():
-    return os.environ.get('TRAVIS') == 'true'
-
-
-@pytest.helpers.register
 def have_unpaper():
     try:
         unpaper.version()
@@ -71,31 +44,35 @@ def have_unpaper():
 
 TESTS_ROOT = Path(__file__).parent.resolve()
 PROJECT_ROOT = TESTS_ROOT
-OCRMYPDF = [sys.executable, '-m', 'ocrmypdf']
 
 
 @pytest.fixture
-def resources():
+def resources() -> Path:
     return Path(TESTS_ROOT) / 'resources'
 
 
 @pytest.fixture
-def ocrmypdf_exec():
-    return OCRMYPDF
+def ocrmypdf_exec() -> List[str]:
+    return [sys.executable, '-m', 'ocrmypdf']
 
 
 @pytest.fixture(scope="function")
-def outdir(tmp_path):
+def outdir(tmp_path) -> Path:
     return tmp_path
 
 
 @pytest.fixture(scope="function")
-def outpdf(tmp_path):
+def outpdf(tmp_path) -> Path:
     return tmp_path / 'out.pdf'
 
 
 @pytest.fixture(scope="function")
-def no_outpdf(tmp_path):
+def outtxt(tmp_path) -> Path:
+    return tmp_path / 'out.txt'
+
+
+@pytest.fixture(scope="function")
+def no_outpdf(tmp_path) -> Path:
     """This just documents the fact that a test is not expected to produce
     output. Unfortunately an assertion failure inside a test fixture produces
     an error rather than a test failure, so no testing is done. It's up to
@@ -103,14 +80,13 @@ def no_outpdf(tmp_path):
     return tmp_path / 'no_output.pdf'
 
 
-@pytest.helpers.register
-def check_ocrmypdf(input_file, output_file, *args):
-    """Run ocrmypdf and confirmed that a valid file was created"""
-    args = [str(input_file), str(output_file)] + [
+def check_ocrmypdf(input_file: Path, output_file: Path, *args) -> Path:
+    """Run ocrmypdf and confirm that a valid plausible PDF was created."""
+    api_args = [str(input_file), str(output_file)] + [
         str(arg) for arg in args if arg is not None
     ]
 
-    _parser, options, plugin_manager = get_parser_options_plugins(args=args)
+    _parser, options, plugin_manager = get_parser_options_plugins(args=api_args)
     api.check_options(options, plugin_manager)
     result = api.run_pipeline(options, plugin_manager=plugin_manager, api=True)
 
@@ -121,56 +97,53 @@ def check_ocrmypdf(input_file, output_file, *args):
     return output_file
 
 
-@pytest.helpers.register
-def run_ocrmypdf_api(input_file, output_file, *args):
-    """Run ocrmypdf via API and let caller deal with results
+def run_ocrmypdf_api(input_file: Path, output_file: Path, *args) -> ExitCode:
+    """Run ocrmypdf via its API in-process, and let test deal with results.
 
-    Does not currently have a way to manipulate the PATH except for Tesseract.
+    This simulates calling the command line interface in a subprocess, but
+    is easier for debuggers and code coverage to follow.
+
+    Any exception raised will be trapped and converted to an exit code.
+    The return code must always be checked or the test may declare a failure
+    to be pass.
     """
 
-    args = [str(input_file), str(output_file)] + [
+    api_args = [str(input_file), str(output_file)] + [
         str(arg) for arg in args if arg is not None
     ]
-    _parser, options, plugin_manager = get_parser_options_plugins(args=args)
+    _parser, options, plugin_manager = get_parser_options_plugins(args=api_args)
 
     api.check_options(options, plugin_manager)
     return api.run_pipeline(options, plugin_manager=None, api=False)
 
 
-@pytest.helpers.register
-def run_ocrmypdf(input_file, output_file, *args, universal_newlines=True):
-    "Run ocrmypdf and let caller deal with results"
+def run_ocrmypdf(
+    input_file: Path, output_file: Path, *args, text: bool = True
+) -> CompletedProcess:
+    """Run ocrmypdf in a subprocess and let test deal with results.
+
+    If an exception is thrown this fact will be returned as part of the result
+    text and return code rather than exception objects.
+    """
 
     p_args = (
-        OCRMYPDF
+        [sys.executable, '-m', 'ocrmypdf']
         + [str(arg) for arg in args if arg is not None]
         + [str(input_file), str(output_file)]
     )
 
-    # Tell subprocess where to find coverage.py configuration
-    # This has no unless except when coverage is running
-    # Details: https://coverage.readthedocs.io/en/coverage-5.0/subprocess.html
-    coverage_rc = Path(__file__).parent.parent / '.coveragerc'
-    env = os.environ.copy()
-    if coverage_rc.exists():
-        env['COVERAGE_PROCESS_START'] = os.fspath(coverage_rc)
-    elif not running_in_docker():
-        assert False, "could not find .coveragerc"
-
     p = run(
         p_args,
         stdout=PIPE,
         stderr=PIPE,
-        universal_newlines=universal_newlines,
-        env=env,
+        text=text,
         check=False,
     )
     # print(p.stderr)
-    return p, p.stdout, p.stderr
+    return p
 
 
-@pytest.helpers.register
-def first_page_dimensions(pdf):
+def first_page_dimensions(pdf: Path):
     info = pdfinfo.PdfInfo(pdf)
     page0 = info[0]
     return (page0.width_inches, page0.height_inches)
