@@ -1,26 +1,19 @@
 # © 2019 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+
+import os
 from subprocess import PIPE, run
 
 import pytest
 
+from .conftest import running_in_docker
+
 pytestmark = pytest.mark.skipif(
-    pytest.helpers.running_in_docker(),  # pylint: disable=no-member
+    running_in_docker(),
     reason="docker can't complete",
 )
 
@@ -39,6 +32,9 @@ def test_fish():
         pytest.xfail('fish is not installed')
 
 
+@pytest.mark.skipif(
+    os.name == 'nt', reason="Windows CI workers have bash but are best left alone"
+)
 def test_bash():
     try:
         proc = run(
