@@ -1,19 +1,9 @@
 # © 2019 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 import logging
 import multiprocessing
@@ -22,8 +12,11 @@ from unittest.mock import MagicMock
 
 import pytest
 
-import ocrmypdf.helpers as helpers
-from ocrmypdf.subprocess import shim_paths_with_program_files
+from ocrmypdf import helpers
+
+from .conftest import running_in_docker
+
+needs_symlink = pytest.mark.skipif(os.name == 'nt', reason='needs posix symlink')
 
 
 class TestSafeSymlink:
@@ -36,6 +29,7 @@ class TestSafeSymlink:
         with pytest.raises(FileExistsError):
             helpers.safe_symlink(tmp_path / 'input', tmp_path / 'regular_file')
 
+    @needs_symlink
     def test_safe_symlink_relink(self, tmp_path):
         (tmp_path / 'regular_file_a').touch()
         (tmp_path / 'regular_file_b').write_bytes(b'ABC')
@@ -47,12 +41,17 @@ class TestSafeSymlink:
 
 
 def test_no_cpu_count(monkeypatch):
+    invoked = False
+
     def cpu_count_raises():
+        nonlocal invoked
+        invoked = True
         raise NotImplementedError()
 
     monkeypatch.setattr(multiprocessing, 'cpu_count', cpu_count_raises)
     with pytest.warns(expected_warning=UserWarning):
         assert helpers.available_cpu_count() == 1
+    assert invoked, "Patched function called during test"
 
 
 def test_deprecated():
@@ -64,9 +63,7 @@ def test_deprecated():
         assert old_function() == 42
 
 
-skipif_docker = pytest.mark.skipif(
-    pytest.helpers.running_in_docker(), reason="fails on Docker"
-)
+skipif_docker = pytest.mark.skipif(running_in_docker(), reason="fails on Docker")
 
 
 class TestFileIsWritable:
@@ -83,6 +80,7 @@ class TestFileIsWritable:
     def test_plain(self, non_existent):
         assert helpers.is_file_writable(non_existent)
 
+    @needs_symlink
     def test_symlink_loop(self, tmp_path):
         loop = tmp_path / 'loop'
         loop.symlink_to(loop)
@@ -104,7 +102,11 @@ class TestFileIsWritable:
         assert not helpers.is_file_writable(pathmock)
 
 
+@pytest.mark.skipif(os.name != 'nt', reason="Windows test")
 def test_shim_paths(tmp_path):
+    # pylint: disable=import-outside-toplevel
+    from ocrmypdf.subprocess._windows import shim_env_path
+
     progfiles = tmp_path / 'Program Files'
     progfiles.mkdir()
     (progfiles / 'tesseract-ocr').mkdir()
@@ -113,9 +115,20 @@ def test_shim_paths(tmp_path):
     syspath = tmp_path / 'bin'
     env = {'PROGRAMFILES': str(progfiles), 'PATH': str(syspath)}
 
-    result_str = shim_paths_with_program_files(env=env)
+    result_str = shim_env_path(env=env)
     results = result_str.split(os.pathsep)
-    assert results[0].endswith('tesseract-ocr')
-    assert results[1].endswith(os.path.join('gs', '9.52', 'bin'))
-    assert results[2].endswith(os.path.join('gs', '9.51', 'bin'))
-    assert results[3] == str(syspath)
+    assert results[0] == str(syspath), results
+    assert results[-3].endswith('tesseract-ocr'), results
+    assert results[-2].endswith(os.path.join('gs', '9.52', 'bin')), results
+    assert results[-1].endswith(os.path.join('gs', '9.51', 'bin')), results
+
+
+def test_resolution():
+    Resolution = helpers.Resolution
+    dpi_100 = Resolution(100, 100)
+    dpi_200 = Resolution(200, 200)
+    assert dpi_100.is_square
+    assert not Resolution(100, 200).is_square
+    assert dpi_100 == Resolution(100, 100)
+    assert str(dpi_100) != str(dpi_200)
+    assert dpi_100.take_max([200, 300], [400]) == Resolution(300, 400)
