@@ -1,19 +1,9 @@
 # © 2019 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 from unittest.mock import patch
 
@@ -24,8 +14,9 @@ from PIL import Image
 
 import ocrmypdf
 
-check_ocrmypdf = pytest.helpers.check_ocrmypdf
-run_ocrmypdf_api = pytest.helpers.run_ocrmypdf_api
+from .conftest import check_ocrmypdf, run_ocrmypdf_api
+
+# pylint: disable=redefined-outer-name
 
 
 @pytest.fixture
@@ -75,13 +66,15 @@ def test_cmyk_no_icc(caplog, resources, no_outpdf):
 def test_img2pdf_fails(resources, no_outpdf):
     with patch(
         'ocrmypdf._pipeline.img2pdf.convert', side_effect=img2pdf.ImageOpenError()
-    ):
+    ) as mock:
         rc = run_ocrmypdf_api(
             resources / 'baiona_gray.png', no_outpdf, '--image-dpi', '200'
         )
         assert rc == ocrmypdf.ExitCode.input_file
+        mock.assert_called()
 
 
+@pytest.mark.xfail(reason="remove background disabled")
 def test_jpeg_in_jpeg_out(resources, outpdf):
     check_ocrmypdf(
         resources / 'congress.jpg',
@@ -95,4 +88,4 @@ def test_jpeg_in_jpeg_out(resources, outpdf):
         'tests/plugins/tesseract_noop.py',
     )
     with pikepdf.open(outpdf) as pdf:
-        assert next(pdf.pages[0].images.values()).Filter == pikepdf.Name.DCTDecode
+        assert next(iter(pdf.pages[0].images.values())).Filter == pikepdf.Name.DCTDecode
