@@ -1,19 +1,9 @@
 # © 2015-19 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 import os
 import shutil
@@ -28,39 +18,36 @@ import pytest
 from PIL import Image
 
 import ocrmypdf
+from ocrmypdf._exec import ghostscript, tesseract
 from ocrmypdf.exceptions import ExitCode, MissingDependencyError
-from ocrmypdf.exec import ghostscript, qpdf, tesseract
 from ocrmypdf.pdfa import file_claims_pdfa
 from ocrmypdf.pdfinfo import Colorspace, Encoding, PdfInfo
+from ocrmypdf.subprocess import get_version
 
-# pytest.helpers is dynamic
-# pylint: disable=no-member,redefined-outer-name
+from .conftest import (
+    check_ocrmypdf,
+    first_page_dimensions,
+    have_unpaper,
+    is_macos,
+    run_ocrmypdf,
+    run_ocrmypdf_api,
+    running_in_docker,
+)
 
-check_ocrmypdf = pytest.helpers.check_ocrmypdf
-run_ocrmypdf = pytest.helpers.run_ocrmypdf
-run_ocrmypdf_api = pytest.helpers.run_ocrmypdf_api
-spoof = pytest.helpers.spoof
+# pylint: disable=redefined-outer-name
 
 
 RENDERERS = ['hocr', 'sandwich']
 
 
-@pytest.fixture
-def spoof_tesseract_crash(tmp_path_factory):
-    return spoof(tmp_path_factory, tesseract='tesseract_crash.py')
-
-
-@pytest.fixture
-def spoof_tesseract_big_image_error(tmp_path_factory):
-    return spoof(tmp_path_factory, tesseract='tesseract_big_image_error.py')
-
-
-def test_quick(spoof_tesseract_cache, resources, outpdf):
-    check_ocrmypdf(resources / 'ccitt.pdf', outpdf, env=spoof_tesseract_cache)
+def test_quick(resources, outpdf):
+    check_ocrmypdf(
+        resources / 'ccitt.pdf', outpdf, '--plugin', 'tests/plugins/tesseract_cache.py'
+    )
 
 
 @pytest.mark.parametrize('renderer', RENDERERS)
-def test_oversample(spoof_tesseract_cache, renderer, resources, outpdf):
+def test_oversample(renderer, resources, outpdf):
     oversampled_pdf = check_ocrmypdf(
         resources / 'skew.pdf',
         outpdf,
@@ -69,13 +56,14 @@ def test_oversample(spoof_tesseract_cache, renderer, resources, outpdf):
         '-f',
         '--pdf-renderer',
         renderer,
-        env=spoof_tesseract_cache,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
 
     pdfinfo = PdfInfo(oversampled_pdf)
 
-    print(pdfinfo[0].xres)
-    assert abs(pdfinfo[0].xres - 350) < 1
+    print(pdfinfo[0].dpi.x)
+    assert abs(pdfinfo[0].dpi.x - 350) < 1
 
 
 def test_repeat_ocr(resources, no_outpdf):
@@ -83,17 +71,25 @@ def test_repeat_ocr(resources, no_outpdf):
     assert result == ExitCode.already_done_ocr
 
 
-def test_force_ocr(spoof_tesseract_cache, resources, outpdf):
+def test_force_ocr(resources, outpdf):
     out = check_ocrmypdf(
-        resources / 'graph_ocred.pdf', outpdf, '-f', env=spoof_tesseract_cache
+        resources / 'graph_ocred.pdf',
+        outpdf,
+        '-f',
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
     pdfinfo = PdfInfo(out)
     assert pdfinfo[0].has_text
 
 
-def test_skip_ocr(spoof_tesseract_cache, resources, outpdf):
+def test_skip_ocr(resources, outpdf):
     out = check_ocrmypdf(
-        resources / 'graph_ocred.pdf', outpdf, '-s', env=spoof_tesseract_cache
+        resources / 'graph_ocred.pdf',
+        outpdf,
+        '-s',
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
     pdfinfo = PdfInfo(out)
     assert pdfinfo[0].has_text
@@ -101,17 +97,17 @@ def test_skip_ocr(spoof_tesseract_cache, resources, outpdf):
 
 def test_redo_ocr(resources, outpdf):
     in_ = resources / 'graph_ocred.pdf'
-    before = PdfInfo(in_, detailed_page_analysis=True)
+    before = PdfInfo(in_, detailed_analysis=True)
     out = outpdf
     out = check_ocrmypdf(in_, out, '--redo-ocr')
-    after = PdfInfo(out, detailed_page_analysis=True)
+    after = PdfInfo(out, detailed_analysis=True)
     assert before[0].has_text and after[0].has_text
     assert (
         before[0].get_textareas() != after[0].get_textareas()
     ), "Expected text to be different after re-OCR"
 
 
-def test_argsfile(spoof_tesseract_noop, resources, outdir):
+def test_argsfile(resources, outdir):
     path_argsfile = outdir / 'test_argsfile.txt'
     with open(str(path_argsfile), 'w') as argsfile:
         print(
@@ -119,15 +115,14 @@ def test_argsfile(spoof_tesseract_noop, resources, outdir):
             'ArgsFile Test',
             '--author',
             'Test Cases',
+            '--plugin',
+            'tests/plugins/tesseract_noop.py',
             sep='\n',
             end='\n',
             file=argsfile,
         )
     check_ocrmypdf(
-        resources / 'graph.pdf',
-        path_argsfile,
-        '@' + str(outdir / 'test_argsfile.txt'),
-        env=spoof_tesseract_noop,
+        resources / 'graph.pdf', path_argsfile, '@' + str(outdir / 'test_argsfile.txt')
     )
 
 
@@ -145,9 +140,14 @@ def test_ocr_timeout(renderer, resources, outpdf):
     assert not pdfinfo[0].has_text
 
 
-def test_skip_big(spoof_tesseract_cache, resources, outpdf):
+def test_skip_big(resources, outpdf):
     out = check_ocrmypdf(
-        resources / 'jbig2.pdf', outpdf, '--skip-big', '1', env=spoof_tesseract_cache
+        resources / 'jbig2.pdf',
+        outpdf,
+        '--skip-big',
+        '1',
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
     pdfinfo = PdfInfo(out)
     assert not pdfinfo[0].has_text
@@ -155,19 +155,16 @@ def test_skip_big(spoof_tesseract_cache, resources, outpdf):
 
 @pytest.mark.parametrize('renderer', RENDERERS)
 @pytest.mark.parametrize('output_type', ['pdf', 'pdfa'])
-def test_maximum_options(
-    spoof_tesseract_cache, renderer, output_type, resources, outpdf
-):
+def test_maximum_options(renderer, output_type, resources, outpdf):
     check_ocrmypdf(
         resources / 'multipage.pdf',
         outpdf,
         '-d',
-        '-ci' if pytest.helpers.have_unpaper() else None,
+        '-ci' if have_unpaper() else None,
         '-f',
         '-k',
         '--oversample',
         '300',
-        '--remove-background',
         '--skip-big',
         '10',
         '--title',
@@ -178,18 +175,18 @@ def test_maximum_options(
         renderer,
         '--output-type',
         output_type,
-        env=spoof_tesseract_cache,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
 
 
-def test_tesseract_missing_tessdata(resources, no_outpdf, tmpdir):
-    env = os.environ.copy()
-    env['TESSDATA_PREFIX'] = os.fspath(tmpdir)
-
-    returncode = run_ocrmypdf_api(
-        resources / 'graph.pdf', no_outpdf, '-v', '1', '--skip-text', env=env
-    )
-    assert returncode == ExitCode.missing_dependency
+@pytest.mark.skipif(
+    tesseract.version() >= '5', reason="tess 5 tries harder to find its files"
+)
+def test_tesseract_missing_tessdata(monkeypatch, resources, no_outpdf, tmpdir):
+    monkeypatch.setenv("TESSDATA_PREFIX", os.fspath(tmpdir))
+    with pytest.raises(MissingDependencyError):
+        run_ocrmypdf_api(resources / 'graph.pdf', no_outpdf, '-v', '1', '--skip-text')
 
 
 def test_invalid_input_pdf(resources, no_outpdf):
@@ -202,22 +199,26 @@ def test_blank_input_pdf(resources, outpdf):
     assert result == ExitCode.ok
 
 
-def test_force_ocr_on_pdf_with_no_images(spoof_tesseract_crash, resources, no_outpdf):
+def test_force_ocr_on_pdf_with_no_images(resources, no_outpdf):
     # As a correctness test, make sure that --force-ocr on a PDF with no
     # content still triggers tesseract. If tesseract crashes, then it was
     # called.
-    p, _, _ = run_ocrmypdf(
-        resources / 'blank.pdf', no_outpdf, '--force-ocr', env=spoof_tesseract_crash
+    p = run_ocrmypdf(
+        resources / 'blank.pdf',
+        no_outpdf,
+        '--force-ocr',
+        '--plugin',
+        'tests/plugins/tesseract_crash.py',
     )
     assert p.returncode == ExitCode.child_process_error
-    assert not os.path.exists(no_outpdf)
+    assert not no_outpdf.exists()
 
 
 @pytest.mark.skipif(
-    pytest.helpers.is_macos() and pytest.helpers.running_in_travis(),
-    reason="takes too long to install language packs in Travis macOS homebrew",
+    is_macos(),
+    reason="takes too long to install language packs in macOS homebrew",
 )
-def test_german(spoof_tesseract_cache, resources, outdir):
+def test_german(resources, outdir):
     # Produce a sidecar too - implicit test that system locale is set up
     # properly. It is fine that we are testing -l deu on a French file because
     # we are exercising the functionality not going for accuracy.
@@ -230,36 +231,41 @@ def test_german(spoof_tesseract_cache, resources, outdir):
             'deu',  # more commonly installed
             '--sidecar',
             sidecar,
-            env=spoof_tesseract_cache,
+            '--plugin',
+            'tests/plugins/tesseract_cache.py',
         )
     except MissingDependencyError:
-        if 'deu' not in tesseract.languages():
+        if 'deu' not in tesseract.get_languages():
             pytest.xfail(reason="tesseract-deu language pack not installed")
         raise
 
 
 def test_klingon(resources, outpdf):
-    p, _, _ = run_ocrmypdf(resources / 'francais.pdf', outpdf, '-l', 'klz')
+    p = run_ocrmypdf(resources / 'francais.pdf', outpdf, '-l', 'klz')
     assert p.returncode == ExitCode.missing_dependency
 
 
-def test_missing_docinfo(spoof_tesseract_noop, resources, outpdf):
+def test_missing_docinfo(resources, outpdf):
     result = run_ocrmypdf_api(
         resources / 'missing_docinfo.pdf',
         outpdf,
         '-l',
         'eng',
         '--skip-text',
-        env=spoof_tesseract_noop,
+        '--plugin',
+        Path('tests/plugins/tesseract_noop.py'),
     )
     assert result == ExitCode.ok
 
 
-def test_uppercase_extension(spoof_tesseract_noop, resources, outdir):
+def test_uppercase_extension(resources, outdir):
     shutil.copy(str(resources / "skew.pdf"), str(outdir / "UPPERCASE.PDF"))
 
     check_ocrmypdf(
-        outdir / "UPPERCASE.PDF", outdir / "UPPERCASE_OUT.PDF", env=spoof_tesseract_noop
+        outdir / "UPPERCASE.PDF",
+        outdir / "UPPERCASE_OUT.PDF",
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
 
 
@@ -270,7 +276,7 @@ def test_input_file_not_found(caplog, no_outpdf):
     assert input_file in caplog.text
 
 
-@pytest.mark.skipif(os.name == 'nt', reason="chmod")
+@pytest.mark.skipif(os.name == 'nt' or running_in_docker(), reason="chmod")
 def test_input_file_not_readable(caplog, resources, outdir, no_outpdf):
     input_file = outdir / 'trivial.pdf'
     shutil.copy(resources / 'trivial.pdf', input_file)
@@ -295,7 +301,7 @@ def test_encrypted(resources, caplog, no_outpdf):
 
 
 @pytest.mark.parametrize('renderer', RENDERERS)
-def test_pagesegmode(renderer, spoof_tesseract_cache, resources, outpdf):
+def test_pagesegmode(renderer, resources, outpdf):
     check_ocrmypdf(
         resources / 'skew.pdf',
         outpdf,
@@ -305,42 +311,82 @@ def test_pagesegmode(renderer, spoof_tesseract_cache, resources, outpdf):
         '1',
         '--pdf-renderer',
         renderer,
-        env=spoof_tesseract_cache,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
 
 
+def test_tesseract_oem(resources, outpdf):
+    check_ocrmypdf(
+        resources / 'trivial.pdf',
+        outpdf,
+        '--tesseract-oem',
+        '1',
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
+    )
+
+
+@pytest.mark.parametrize('value', ['auto', 'otsu', 'adaptive-otsu', 'sauvola'])
+def test_tesseract_thresholding(value, resources, outpdf):
+    check_ocrmypdf(
+        resources / 'trivial.pdf',
+        outpdf,
+        '--tesseract-thresholding',
+        value,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
+    )
+
+
+@pytest.mark.parametrize('value', ['abcxyz'])
+def test_tesseract_thresholding_invalid(value, resources, no_outpdf):
+    with pytest.raises(SystemExit, match='2'):
+        run_ocrmypdf_api(
+            resources / 'trivial.pdf',
+            no_outpdf,
+            '--tesseract-thresholding',
+            value,
+            '--plugin',
+            'tests/plugins/tesseract_cache.py',
+        )
+
+
 @pytest.mark.parametrize('renderer', RENDERERS)
-def test_tesseract_crash(renderer, spoof_tesseract_crash, resources, no_outpdf, caplog):
-    p, _, err = run_ocrmypdf(
+def test_tesseract_crash(renderer, resources, no_outpdf):
+    p = run_ocrmypdf(
         resources / 'ccitt.pdf',
         no_outpdf,
         '-v',
         '1',
         '--pdf-renderer',
         renderer,
-        env=spoof_tesseract_crash,
+        '--plugin',
+        'tests/plugins/tesseract_crash.py',
     )
     assert p.returncode == ExitCode.child_process_error
-    assert not os.path.exists(no_outpdf)
-    assert "SubprocessOutputError" in err
+    assert not no_outpdf.exists()
+    assert "SubprocessOutputError" in p.stderr
 
 
-def test_tesseract_crash_autorotate(spoof_tesseract_crash, resources, no_outpdf):
-    p, out, err = run_ocrmypdf(
-        resources / 'ccitt.pdf', no_outpdf, '-r', env=spoof_tesseract_crash
+def test_tesseract_crash_autorotate(resources, no_outpdf):
+    p = run_ocrmypdf(
+        resources / 'ccitt.pdf',
+        no_outpdf,
+        '-r',
+        '--plugin',
+        'tests/plugins/tesseract_crash.py',
     )
     assert p.returncode == ExitCode.child_process_error
-    assert not os.path.exists(no_outpdf)
-    assert "ERROR" in err
-    print(out)
-    print(err)
+    assert not no_outpdf.exists()
+    assert "uncaught exception" in p.stderr
+    print(p.stdout)
+    print(p.stderr)
 
 
 @pytest.mark.parametrize('renderer', RENDERERS)
 @pytest.mark.slow
-def test_tesseract_image_too_big(
-    renderer, spoof_tesseract_big_image_error, resources, outpdf
-):
+def test_tesseract_image_too_big(renderer, resources, outpdf):
     check_ocrmypdf(
         resources / 'hugemono.pdf',
         outpdf,
@@ -349,18 +395,22 @@ def test_tesseract_image_too_big(
         renderer,
         '--max-image-mpixels',
         '0',
-        env=spoof_tesseract_big_image_error,
+        '--plugin',
+        'tests/plugins/tesseract_big_image_error.py',
     )
 
 
-def test_algo4(resources, spoof_tesseract_noop, outpdf):
-    p, _, _ = run_ocrmypdf(
-        resources / 'encrypted_algo4.pdf', outpdf, env=spoof_tesseract_noop
+def test_algo4(resources, outpdf):
+    p = run_ocrmypdf(
+        resources / 'encrypted_algo4.pdf',
+        outpdf,
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
     assert p.returncode == ExitCode.encrypted_pdf
 
 
-def test_jbig2_passthrough(spoof_tesseract_cache, resources, outpdf):
+def test_jbig2_passthrough(resources, outpdf):
     out = check_ocrmypdf(
         resources / 'jbig2.pdf',
         outpdf,
@@ -368,49 +418,64 @@ def test_jbig2_passthrough(spoof_tesseract_cache, resources, outpdf):
         'pdf',
         '--pdf-renderer',
         'hocr',
-        env=spoof_tesseract_cache,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
     out_pageinfo = PdfInfo(out)
     assert out_pageinfo[0].images[0].enc == Encoding.jbig2
 
 
-def test_masks(spoof_tesseract_noop, resources, outpdf):
+def test_masks(resources, outpdf):
     assert (
         ocrmypdf.ocr(
-            resources / 'masks.pdf', outpdf, tesseract_env=spoof_tesseract_noop
+            resources / 'masks.pdf', outpdf, plugins=['tests/plugins/tesseract_noop.py']
         )
         == ExitCode.ok
     )
 
 
-def test_linearized_pdf_and_indirect_object(spoof_tesseract_noop, resources, outpdf):
-    check_ocrmypdf(resources / 'epson.pdf', outpdf, env=spoof_tesseract_noop)
-
-
-def test_very_high_dpi(spoof_tesseract_cache, resources, outpdf):
-    "Checks for a Decimal quantize error with high DPI, etc"
-    check_ocrmypdf(resources / '2400dpi.pdf', outpdf, env=spoof_tesseract_cache)
-    pdfinfo = PdfInfo(outpdf)
-
-    image = pdfinfo[0].images[0]
-    assert isclose(image.xres, image.yres)
-    assert isclose(image.xres, 2400)
-
-
-def test_overlay(spoof_tesseract_noop, resources, outpdf):
+def test_linearized_pdf_and_indirect_object(resources, outpdf):
     check_ocrmypdf(
-        resources / 'overlay.pdf', outpdf, '--skip-text', env=spoof_tesseract_noop
+        resources / 'epson.pdf', outpdf, '--plugin', 'tests/plugins/tesseract_noop.py'
     )
 
 
-def test_destination_not_writable(spoof_tesseract_noop, resources, outdir):
+def test_very_high_dpi(resources, outpdf):
+    "Checks for a Decimal quantize error with high DPI, etc"
+    check_ocrmypdf(
+        resources / '2400dpi.pdf',
+        outpdf,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
+    )
+    pdfinfo = PdfInfo(outpdf)
+
+    image = pdfinfo[0].images[0]
+    assert isclose(image.dpi.x, image.dpi.y)
+    assert isclose(image.dpi.x, 2400)
+
+
+def test_overlay(resources, outpdf):
+    check_ocrmypdf(
+        resources / 'overlay.pdf',
+        outpdf,
+        '--skip-text',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
+    )
+
+
+def test_destination_not_writable(resources, outdir):
     if os.name != 'nt' and (os.getuid() == 0 or os.geteuid() == 0):
         pytest.xfail(reason="root can write to anything")
     protected_file = outdir / 'protected.pdf'
     protected_file.touch()
     protected_file.chmod(0o400)  # Read-only
-    p, out, err = run_ocrmypdf(
-        resources / 'jbig2.pdf', protected_file, env=spoof_tesseract_noop
+    p = run_ocrmypdf(
+        resources / 'jbig2.pdf',
+        protected_file,
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
     assert p.returncode == ExitCode.file_access_error, "Expected error"
 
@@ -447,7 +512,7 @@ THIS FILE IS INVALID
 '''
         )
 
-    p, out, err = run_ocrmypdf(
+    p = run_ocrmypdf(
         resources / 'ccitt.pdf',
         outdir / 'out.pdf',
         '--pdf-renderer',
@@ -456,8 +521,8 @@ THIS FILE IS INVALID
         cfg_file,
     )
     assert (
-        "parameter not found" in err.lower()
-        or "error occurred while parsing" in err.lower()
+        "parameter not found" in p.stderr.lower()
+        or "error occurred while parsing" in p.stderr.lower()
     ), "No error message"
     assert p.returncode == ExitCode.invalid_config
 
@@ -483,17 +548,18 @@ def test_user_words_ocr(resources, outdir):
     )
 
 
-def test_form_xobject(spoof_tesseract_noop, resources, outpdf):
+def test_form_xobject(resources, outpdf):
     check_ocrmypdf(
-        resources / 'formxobject.pdf', outpdf, '--force-ocr', env=spoof_tesseract_noop
+        resources / 'formxobject.pdf',
+        outpdf,
+        '--force-ocr',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
 
 
 @pytest.mark.parametrize('renderer', RENDERERS)
 def test_pagesize_consistency(renderer, resources, outpdf):
-
-    first_page_dimensions = pytest.helpers.first_page_dimensions
-
     infile = resources / '3small.pdf'
 
     before_dims = first_page_dimensions(infile)
@@ -503,10 +569,11 @@ def test_pagesize_consistency(renderer, resources, outpdf):
         outpdf,
         '--pdf-renderer',
         renderer,
-        '--clean' if pytest.helpers.have_unpaper() else None,
+        '--clean' if have_unpaper() else None,
         '--deskew',
-        '--remove-background',
-        '--clean-final' if pytest.helpers.have_unpaper() else None,
+        # '--remove-background',
+        '--clean-final' if have_unpaper() else None,
+        '-k',
         '--pages',
         '1',
     )
@@ -517,33 +584,32 @@ def test_pagesize_consistency(renderer, resources, outpdf):
     assert isclose(before_dims[1], after_dims[1], rel_tol=1e-4)
 
 
-def test_skip_big_with_no_images(spoof_tesseract_noop, resources, outpdf):
+def test_skip_big_with_no_images(resources, outpdf):
     check_ocrmypdf(
         resources / 'blank.pdf',
         outpdf,
         '--skip-big',
         '5',
         '--force-ocr',
-        env=spoof_tesseract_noop,
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
 
 
-@pytest.mark.skipif(
-    '8.0.0' <= qpdf.version() <= '8.0.1',
-    reason="qpdf regression on pages with no contents",
-)
-def test_no_contents(spoof_tesseract_noop, resources, outpdf):
+def test_no_contents(resources, outpdf):
     check_ocrmypdf(
-        resources / 'no_contents.pdf', outpdf, '--force-ocr', env=spoof_tesseract_noop
+        resources / 'no_contents.pdf',
+        outpdf,
+        '--force-ocr',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
 
 
 @pytest.mark.parametrize(
     'image', ['baiona.png', 'baiona_gray.png', 'baiona_alpha.png', 'congress.jpg']
 )
-def test_compression_preserved(
-    spoof_tesseract_noop, ocrmypdf_exec, resources, image, outpdf
-):
+def test_compression_preserved(ocrmypdf_exec, resources, image, outpdf):
     input_file = str(resources / image)
     output_file = str(outpdf)
 
@@ -557,6 +623,8 @@ def test_compression_preserved(
             '150',
             '--output-type',
             'pdf',
+            '--plugin',
+            'tests/plugins/tesseract_noop.py',
             '-',
             output_file,
         ]
@@ -565,8 +633,8 @@ def test_compression_preserved(
             stdout=PIPE,
             stderr=PIPE,
             stdin=input_stream,
-            universal_newlines=True,
-            env=spoof_tesseract_noop,
+            text=True,
+            check=False,
         )
 
         if im.mode in ('RGBA', 'LA'):
@@ -599,9 +667,7 @@ def test_compression_preserved(
         ('congress.jpg', 'lossless'),
     ],
 )
-def test_compression_changed(
-    spoof_tesseract_noop, ocrmypdf_exec, resources, image, compression, outpdf
-):
+def test_compression_changed(ocrmypdf_exec, resources, image, compression, outpdf):
     input_file = str(resources / image)
     output_file = str(outpdf)
 
@@ -618,6 +684,8 @@ def test_compression_changed(
             '0',
             '--pdfa-image-compression',
             compression,
+            '--plugin',
+            'tests/plugins/tesseract_noop.py',
             '-',
             output_file,
         ]
@@ -626,8 +694,8 @@ def test_compression_changed(
             stdout=PIPE,
             stderr=PIPE,
             stdin=input_stream,
-            universal_newlines=True,
-            env=spoof_tesseract_noop,
+            text=True,
+            check=False,
         )
         assert p.returncode == ExitCode.ok, p.stderr
 
@@ -653,7 +721,7 @@ def test_compression_changed(
     im.close()
 
 
-def test_sidecar_pagecount(spoof_tesseract_cache, resources, outpdf):
+def test_sidecar_pagecount(resources, outpdf):
     sidecar = outpdf.with_suffix('.txt')
     check_ocrmypdf(
         resources / '3small.pdf',
@@ -661,13 +729,14 @@ def test_sidecar_pagecount(spoof_tesseract_cache, resources, outpdf):
         '--skip-text',
         '--sidecar',
         sidecar,
-        env=spoof_tesseract_cache,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
 
     pdfinfo = PdfInfo(resources / '3small.pdf')
     num_pages = len(pdfinfo)
 
-    with open(sidecar, 'r', encoding='utf-8') as f:
+    with open(sidecar, encoding='utf-8') as f:
         ocr_text = f.read()
 
     # There should a formfeed between each pair of pages, so the count of
@@ -677,19 +746,24 @@ def test_sidecar_pagecount(spoof_tesseract_cache, resources, outpdf):
     ), "Sidecar page count does not match PDF page count"
 
 
-def test_sidecar_nonempty(spoof_tesseract_cache, resources, outpdf):
+def test_sidecar_nonempty(resources, outpdf):
     sidecar = outpdf.with_suffix('.txt')
     check_ocrmypdf(
-        resources / 'ccitt.pdf', outpdf, '--sidecar', sidecar, env=spoof_tesseract_cache
+        resources / 'ccitt.pdf',
+        outpdf,
+        '--sidecar',
+        sidecar,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
 
-    with open(sidecar, 'r', encoding='utf-8') as f:
+    with open(sidecar, encoding='utf-8') as f:
         ocr_text = f.read()
     assert 'the' in ocr_text
 
 
 @pytest.mark.parametrize('pdfa_level', ['1', '2', '3'])
-def test_pdfa_n(spoof_tesseract_cache, pdfa_level, resources, outpdf):
+def test_pdfa_n(pdfa_level, resources, outpdf):
     if pdfa_level == '3' and ghostscript.version() < '9.19':
         pytest.xfail(reason='Ghostscript >= 9.19 required')
 
@@ -698,70 +772,83 @@ def test_pdfa_n(spoof_tesseract_cache, pdfa_level, resources, outpdf):
         outpdf,
         '--output-type',
         'pdfa-' + pdfa_level,
-        env=spoof_tesseract_cache,
+        '--plugin',
+        'tests/plugins/tesseract_cache.py',
     )
 
     pdfa_info = file_claims_pdfa(outpdf)
     assert pdfa_info['conformance'] == f'PDF/A-{pdfa_level}B'
 
 
-@pytest.mark.skipif(
-    PIL.__version__ < '5.0.0', reason="Pillow < 5.0.0 doesn't raise the exception"
-)
-@pytest.mark.slow
-def test_decompression_bomb(resources, outpdf):
-    p, out, err = run_ocrmypdf(resources / 'hugemono.pdf', outpdf)
-    assert 'decompression bomb' in err
+def test_decompression_bomb_error(resources, outpdf):
+    p = run_ocrmypdf(resources / 'hugemono.pdf', outpdf)
+    assert 'decompression bomb' in p.stderr and '--max-image-mpixels' in p.stderr
 
-    p, out, err = run_ocrmypdf(
-        resources / 'hugemono.pdf', outpdf, '--max-image-mpixels', '2000'
-    )
+
+@pytest.mark.slow
+def test_decompression_bomb_succeeds(resources, outpdf):
+    p = run_ocrmypdf(resources / 'hugemono.pdf', outpdf, '--max-image-mpixels', '2000')
     assert p.returncode == 0
 
 
-def test_text_curves(spoof_tesseract_noop, resources, outpdf):
+def test_text_curves(resources, outpdf):
     with patch('ocrmypdf._pipeline.VECTOR_PAGE_DPI', 100):
-        check_ocrmypdf(resources / 'vector.pdf', outpdf, env=spoof_tesseract_noop)
+        check_ocrmypdf(
+            resources / 'vector.pdf',
+            outpdf,
+            '--plugin',
+            'tests/plugins/tesseract_noop.py',
+        )
 
         info = PdfInfo(outpdf)
         assert len(info.pages[0].images) == 0, "added images to the vector PDF"
 
         check_ocrmypdf(
-            resources / 'vector.pdf', outpdf, '--force-ocr', env=spoof_tesseract_noop
+            resources / 'vector.pdf',
+            outpdf,
+            '--force-ocr',
+            '--plugin',
+            'tests/plugins/tesseract_noop.py',
         )
 
         info = PdfInfo(outpdf)
         assert len(info.pages[0].images) != 0, "force did not rasterize"
 
 
-def test_output_is_dir(spoof_tesseract_noop, resources, outdir):
-    p, out, err = run_ocrmypdf(
-        resources / 'trivial.pdf', outdir, '--force-ocr', env=spoof_tesseract_noop
+def test_output_is_dir(resources, outdir):
+    p = run_ocrmypdf(
+        resources / 'trivial.pdf',
+        outdir,
+        '--force-ocr',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
     assert p.returncode == ExitCode.file_access_error
-    assert 'is not a writable file' in err
+    assert 'is not a writable file' in p.stderr
 
 
 @pytest.mark.skipif(os.name == 'nt', reason="symlink needs admin permissions")
-def test_output_is_symlink(spoof_tesseract_noop, resources, outdir):
+def test_output_is_symlink(resources, outdir):
     sym = Path(outdir / 'this_is_a_symlink')
     sym.symlink_to(outdir / 'out.pdf')
-    p, out, err = run_ocrmypdf(
-        resources / 'trivial.pdf', sym, '--force-ocr', env=spoof_tesseract_noop
+    p = run_ocrmypdf(
+        resources / 'trivial.pdf',
+        sym,
+        '--force-ocr',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
-    assert p.returncode == ExitCode.ok, err
+    assert p.returncode == ExitCode.ok, p.stderr
     assert (outdir / 'out.pdf').stat().st_size > 0, 'target file not created'
 
 
 def test_livecycle(resources, no_outpdf):
-    p, _, err = run_ocrmypdf(resources / 'livecycle.pdf', no_outpdf)
+    p = run_ocrmypdf(resources / 'livecycle.pdf', no_outpdf)
 
-    assert p.returncode == ExitCode.input_file, err
+    assert p.returncode == ExitCode.input_file, p.stderr
 
 
 def test_version_check():
-    from ocrmypdf.exec import get_version
-
     with pytest.raises(MissingDependencyError):
         get_version('NOT_FOUND_UNLIKELY_ON_PATH')
 
@@ -785,9 +872,7 @@ def test_version_check():
         [0.0, 1, 'pdf', True],
     ],
 )
-def test_fast_web_view(
-    spoof_tesseract_noop, resources, outpdf, threshold, optimize, output_type, expected
-):
+def test_fast_web_view(resources, outpdf, threshold, optimize, output_type, expected):
     check_ocrmypdf(
         resources / 'trivial.pdf',
         outpdf,
@@ -797,18 +882,45 @@ def test_fast_web_view(
         optimize,
         '--output-type',
         output_type,
-        env=spoof_tesseract_noop,
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
     with pikepdf.open(outpdf) as pdf:
         assert pdf.is_linearized == expected
 
 
-def test_image_dpi_not_image(caplog, spoof_tesseract_noop, resources, outpdf):
+def test_image_dpi_not_image(caplog, resources, outpdf):
     check_ocrmypdf(
         resources / 'trivial.pdf',
         outpdf,
         '--image-dpi',
         '100',
-        env=spoof_tesseract_noop,
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
     )
     assert '--image-dpi is being ignored' in caplog.text
+
+
+def test_outputtype_none_bad_setup(resources, outpdf):
+    p = run_ocrmypdf(
+        resources / 'trivial.pdf',
+        outpdf,
+        '--output-type=none',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
+    )
+    assert p.returncode == ExitCode.bad_args
+    assert 'Set the output file to' in p.stderr
+
+
+def test_outputtype_none(resources, outtxt):
+    p = run_ocrmypdf(
+        resources / 'trivial.pdf',
+        '-',
+        '--output-type=none',
+        '--sidecar',
+        outtxt,
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
+    )
+    assert p.returncode == ExitCode.ok
