@@ -1,23 +1,11 @@
 # © 2018 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 import datetime
-import mmap
 from datetime import timezone
 from os import fspath
 from shutil import copyfile
@@ -32,22 +20,19 @@ from ocrmypdf._pipeline import convert_to_pdfa, metadata_fixup
 from ocrmypdf._plugin_manager import get_plugin_manager
 from ocrmypdf.cli import get_parser
 from ocrmypdf.exceptions import ExitCode
-from ocrmypdf.pdfa import SRGB_ICC_PROFILE, file_claims_pdfa, generate_pdfa_ps
+from ocrmypdf.pdfa import file_claims_pdfa, generate_pdfa_ps
 from ocrmypdf.pdfinfo import PdfInfo
 
+from .conftest import check_ocrmypdf, run_ocrmypdf
+
 try:
     import fitz
 except ImportError:
     fitz = None
 
-# pytest.helpers is dynamic
-# pylint: disable=no-member
 
 pytestmark = pytest.mark.filterwarnings('ignore:.*XMLParser.*:DeprecationWarning')
 
-check_ocrmypdf = pytest.helpers.check_ocrmypdf
-run_ocrmypdf = pytest.helpers.run_ocrmypdf
-
 
 @pytest.mark.parametrize("output_type", ['pdfa', 'pdf'])
 def test_preserve_docinfo(output_type, resources, outpdf):
@@ -77,7 +62,7 @@ def test_override_metadata(output_type, resources, outpdf):
     german = 'Du siehst den Wald vor lauter Bäumen nicht.'
     chinese = '孔子'
 
-    p, _out, err = run_ocrmypdf(
+    p = run_ocrmypdf(
         input_file,
         outpdf,
         '--title',
@@ -90,7 +75,7 @@ def test_override_metadata(output_type, resources, outpdf):
         'tests/plugins/tesseract_noop.py',
     )
 
-    assert p.returncode == ExitCode.ok, err
+    assert p.returncode == ExitCode.ok, p.stderr
 
     before = pikepdf.open(input_file)
     after = pikepdf.open(outpdf)
@@ -114,7 +99,7 @@ def test_high_unicode(resources, no_outpdf):
     input_file = resources / 'c02-22.pdf'
     high_unicode = 'U+1030C is: 𐌌'
 
-    p, _out, err = run_ocrmypdf(
+    p = run_ocrmypdf(
         input_file,
         no_outpdf,
         '--subject',
@@ -125,7 +110,7 @@ def test_high_unicode(resources, no_outpdf):
         'tests/plugins/tesseract_noop.py',
     )
 
-    assert p.returncode == ExitCode.bad_args, err
+    assert p.returncode == ExitCode.bad_args, p.stderr
 
 
 @pytest.mark.skipif(not fitz, reason="test uses fitz")
@@ -133,7 +118,7 @@ def test_high_unicode(resources, no_outpdf):
 @pytest.mark.parametrize('output_type', ['pdf', 'pdfa'])
 def test_bookmarks_preserved(output_type, ocr_option, resources, outpdf):
     input_file = resources / 'toc.pdf'
-    before_toc = fitz.Document(str(input_file)).getToC()
+    before_toc = fitz.Document(str(input_file)).get_toc()
 
     check_ocrmypdf(
         input_file,
@@ -145,7 +130,7 @@ def test_bookmarks_preserved(output_type, ocr_option, resources, outpdf):
         'tests/plugins/tesseract_noop.py',
     )
 
-    after_toc = fitz.Document(str(outpdf)).getToC()
+    after_toc = fitz.Document(str(outpdf)).get_toc()
     print(before_toc)
     print(after_toc)
     assert before_toc == after_toc
@@ -288,21 +273,8 @@ def test_xml_metadata_preserved(test_file, output_type, resources, outpdf):
             )
 
 
-def test_srgb_in_unicode_path(tmp_path):
-    """Test that we can produce pdfmark when install path is not ASCII"""
-
-    dstdir = tmp_path / b'\xe4\x80\x80'.decode('utf-8')
-    dstdir.mkdir()
-    dst = dstdir / 'sRGB.icc'
-
-    copyfile(SRGB_ICC_PROFILE, fspath(dst))
-
-    with patch('ocrmypdf.pdfa.SRGB_ICC_PROFILE', new=str(dst)):
-        generate_pdfa_ps(dstdir / 'out.ps')
-
-
 def test_kodak_toc(resources, outpdf):
-    _output = check_ocrmypdf(
+    check_ocrmypdf(
         resources / 'kcs.pdf',
         outpdf,
         '--output-type',
@@ -313,8 +285,8 @@ def test_kodak_toc(resources, outpdf):
 
     p = pikepdf.open(outpdf)
 
-    if pikepdf.Name.First in p.root.Outlines:
-        assert isinstance(p.root.Outlines.First, pikepdf.Dictionary)
+    if pikepdf.Name.First in p.Root.Outlines:
+        assert isinstance(p.Root.Outlines.First, pikepdf.Dictionary)
 
 
 def test_metadata_fixup_warning(resources, outdir, caplog):
@@ -329,7 +301,7 @@ def test_metadata_fixup_warning(resources, outdir, caplog):
     )
     metadata_fixup(working_file=outdir / 'graph.pdf', context=context)
     for record in caplog.records:
-        assert record.levelname != 'WARNING'
+        assert record.levelname != 'WARNING', "Unexpected warning"
 
     # Now add some metadata that will not be copyable
     graph = pikepdf.open(outdir / 'graph.pdf')
@@ -367,17 +339,16 @@ def test_prevent_gs_invalid_xml(resources, outdir):
         str(outdir / 'layers.rendered.pdf'), str(outdir / 'pdfa.ps'), context
     )
 
-    with open(outdir / 'pdfa.pdf', 'r+b') as f:
-        with mmap.mmap(f.fileno(), 0) as mm:
-            # Since the XML may be invalid, we scan instead of actually feeding it
-            # to a parser.
-            XMP_MAGIC = b'W5M0MpCehiHzreSzNTczkc9d'
-            xmp_start = mm.find(XMP_MAGIC)
-            xmp_end = mm.rfind(b'<?xpacket end', xmp_start)
-            assert 0 < xmp_start < xmp_end
-            # Ensure we did not carry the nul forward.
-            assert mm.find(b'&#0;', xmp_start, xmp_end) == -1, "found escaped nul"
-            assert mm.find(b'\x00', xmp_start, xmp_end) == -1
+    contents = (outdir / 'pdfa.pdf').read_bytes()
+    # Since the XML may be invalid, we scan instead of actually feeding it
+    # to a parser.
+    XMP_MAGIC = b'W5M0MpCehiHzreSzNTczkc9d'
+    xmp_start = contents.find(XMP_MAGIC)
+    xmp_end = contents.rfind(b'<?xpacket end', xmp_start)
+    assert 0 < xmp_start < xmp_end
+    # Ensure we did not carry the nul forward.
+    assert contents.find(b'&#0;', xmp_start, xmp_end) == -1, "found escaped nul"
+    assert contents.find(b'\x00', xmp_start, xmp_end) == -1
 
 
 def test_malformed_docinfo(caplog, resources, outdir):
