@@ -1,19 +1,9 @@
 # © 2018 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 from os import fspath
 from pathlib import Path
@@ -27,11 +17,19 @@ from PIL import Image, ImageDraw
 from ocrmypdf import optimize as opt
 from ocrmypdf._exec import jbig2enc, pngquant
 from ocrmypdf._exec.ghostscript import rasterize_pdf
-from ocrmypdf.helpers import Resolution
+from ocrmypdf.helpers import IMG2PDF_KWARGS, Resolution
 
-check_ocrmypdf = pytest.helpers.check_ocrmypdf  # pylint: disable=e1101
+from .conftest import check_ocrmypdf
+
+needs_pngquant = pytest.mark.skipif(
+    not pngquant.available(), reason="pngquant not installed"
+)
+needs_jbig2enc = pytest.mark.skipif(
+    not jbig2enc.available(), reason="jbig2enc not installed"
+)
 
 
+@needs_pngquant
 @pytest.mark.parametrize('pdf', ['multipage.pdf', 'palette.pdf'])
 def test_basic(resources, pdf, outpdf):
     infile = resources / pdf
@@ -40,6 +38,7 @@ def test_basic(resources, pdf, outpdf):
     assert 0.98 * Path(outpdf).stat().st_size <= Path(infile).stat().st_size
 
 
+@needs_pngquant
 def test_mono_not_inverted(resources, outdir):
     infile = resources / '2400dpi.pdf'
     opt.main(infile, outdir / 'out.pdf', level=3)
@@ -52,10 +51,10 @@ def test_mono_not_inverted(resources, outdir):
     )
 
     with Image.open(fspath(outdir / 'im.png')) as im:
-        assert im.getpixel((0, 0)) == 255, "Expected white background"
+        assert im.getpixel((0, 0)) > 240, "Expected white background"
 
 
-@pytest.mark.skipif(not pngquant.available(), reason='need pngquant')
+@needs_pngquant
 def test_jpg_png_params(resources, outpdf):
     check_ocrmypdf(
         resources / 'crom.png',
@@ -73,7 +72,7 @@ def test_jpg_png_params(resources, outpdf):
     )
 
 
-@pytest.mark.skipif(not jbig2enc.available(), reason='need jbig2enc')
+@needs_jbig2enc
 @pytest.mark.parametrize('lossy', [False, True])
 def test_jbig2_lossy(lossy, resources, outpdf):
     args = [
@@ -82,7 +81,7 @@ def test_jbig2_lossy(lossy, resources, outpdf):
         '--image-dpi',
         '200',
         '--optimize',
-        3,
+        '3',
         '--jpg-quality',
         '50',
         '--png-quality',
@@ -105,10 +104,8 @@ def test_jbig2_lossy(lossy, resources, outpdf):
         assert len(pim.decode_parms) == 0
 
 
-@pytest.mark.skipif(
-    not jbig2enc.available() or not pngquant.available(),
-    reason='need jbig2enc and pngquant',
-)
+@needs_pngquant
+@needs_jbig2enc
 def test_flate_to_jbig2(resources, outdir):
     # This test requires an image that pngquant is capable of converting to
     # to 1bpp - so use an existing 1bpp image, convert up, confirm it can
@@ -136,22 +133,24 @@ def test_flate_to_jbig2(resources, outdir):
     assert pim.filters[0] == '/JBIG2Decode'
 
 
+@needs_pngquant
 def test_multiple_pngs(resources, outdir):
     with Path.open(outdir / 'in.pdf', 'wb') as inpdf:
         img2pdf.convert(
             fspath(resources / 'baiona_colormapped.png'),
             fspath(resources / 'baiona_gray.png'),
-            with_pdfrw=False,
             outputstream=inpdf,
+            **IMG2PDF_KWARGS,
         )
 
-    def mockquant(input_file, output_file, _quality_min, _quality_max):
+    def mockquant(input_file, output_file, *_args):
         with Image.open(input_file) as im:
             draw = ImageDraw.Draw(im)
             draw.rectangle((0, 0, im.width, im.height), fill=128)
             im.save(output_file)
 
-    with patch('ocrmypdf.optimize.pngquant.quantize', new=mockquant):
+    with patch('ocrmypdf.optimize.pngquant.quantize') as mock:
+        mock.side_effect = mockquant
         check_ocrmypdf(
             outdir / 'in.pdf',
             outdir / 'out.pdf',
@@ -165,6 +164,7 @@ def test_multiple_pngs(resources, outdir):
             '--plugin',
             'tests/plugins/tesseract_noop.py',
         )
+        mock.assert_called()
 
     with pikepdf.open(outdir / 'in.pdf') as inpdf, pikepdf.open(
         outdir / 'out.pdf'
@@ -173,3 +173,28 @@ def test_multiple_pngs(resources, outdir):
             inim = next(iter(inpdf.pages[n].images.values()))
             outim = next(iter(outpdf.pages[n].images.values()))
             assert len(outim.read_raw_bytes()) < len(inim.read_raw_bytes()), n
+
+
+def test_optimize_off(resources, outpdf):
+    check_ocrmypdf(
+        resources / 'trivial.pdf',
+        outpdf,
+        '--optimize=0',
+        '--output-type',
+        'pdf',
+        '--plugin',
+        'tests/plugins/tesseract_noop.py',
+    )
+
+
+def test_group3(resources, outdir):
+    with pikepdf.open(resources / 'ccitt.pdf') as pdf:
+        im = pdf.pages[0].Resources.XObject['/Im1']
+        assert (
+            opt.extract_image_filter(pdf, outdir, im, im.objgen[0]) is not None
+        ), "Group 4 should be allowed"
+
+        im.DecodeParms['/K'] = 0
+        assert (
+            opt.extract_image_filter(pdf, outdir, im, im.objgen[0]) is None
+        ), "Group 3 should be disallowed"
