@@ -1,41 +1,35 @@
 # © 2015 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import pickle
+from io import BytesIO
 from math import isclose
 
 import img2pdf
 import pikepdf
 import pytest
 from PIL import Image
+from reportlab.lib.units import inch
 from reportlab.pdfgen.canvas import Canvas
 
 from ocrmypdf import pdfinfo
+from ocrmypdf.exceptions import InputFileError
+from ocrmypdf.helpers import IMG2PDF_KWARGS, Resolution
 from ocrmypdf.pdfinfo import Colorspace, Encoding
+from ocrmypdf.pdfinfo.layout import PDFPage
 
 # pylint: disable=protected-access
 
 
 def test_single_page_text(outdir):
     filename = outdir / 'text.pdf'
-    pdf = Canvas(str(filename), pagesize=(8 * 72, 6 * 72))
+    pdf = Canvas(str(filename), pagesize=(8 * inch, 6 * inch))
     text = pdf.beginText()
     text.setFont('Helvetica', 12)
-    text.setTextOrigin(1 * 72, 3 * 72)
+    text.setTextOrigin(1 * inch, 3 * inch)
     text.textLine(
         "Methink'st thou art a general offence and every" " man should beat thee."
     )
@@ -52,25 +46,32 @@ def test_single_page_text(outdir):
     assert len(page.images) == 0
 
 
-def test_single_page_image(outdir):
-    filename = outdir / 'image-mono.pdf'
-
-    im_tmp = outdir / 'tmp.png'
+@pytest.fixture(scope='session')
+def eight_by_eight():
     im = Image.new('1', (8, 8), 0)
     for n in range(8):
         im.putpixel((n, n), 1)
-    im.save(str(im_tmp), format='PNG')
+    return im
+
+
+def test_single_page_image(eight_by_eight, outpdf):
+    im = eight_by_eight
+    bio = BytesIO()
+    im.save(bio, format='PNG')
+    bio.seek(0)
 
     imgsize = ((img2pdf.ImgSize.dpi, 8), (img2pdf.ImgSize.dpi, 8))
     layout_fun = img2pdf.get_layout_fun(None, imgsize, None, None, None)
 
-    im_bytes = im_tmp.read_bytes()
-    pdf_bytes = img2pdf.convert(
-        im_bytes, producer="img2pdf", with_pdfrw=False, layout_fun=layout_fun
-    )
-    filename.write_bytes(pdf_bytes)
-
-    info = pdfinfo.PdfInfo(filename)
+    with outpdf.open('wb') as f:
+        img2pdf.convert(
+            bio,
+            producer="img2pdf",
+            layout_fun=layout_fun,
+            outputstream=f,
+            **IMG2PDF_KWARGS,
+        )
+    info = pdfinfo.PdfInfo(outpdf)
 
     assert len(info) == 1
     page = info[0]
@@ -87,16 +88,12 @@ def test_single_page_image(outdir):
     assert isclose(pdfimage.dpi.y, 8)
 
 
-def test_single_page_inline_image(outdir):
+def test_single_page_inline_image(eight_by_eight, outdir):
     filename = outdir / 'image-mono-inline.pdf'
     pdf = Canvas(str(filename), pagesize=(8 * 72, 6 * 72))
 
-    im = Image.new('1', (8, 8), 0)
-    for n in range(8):
-        im.putpixel((n, n), 1)
-
     # Draw image in a 72x72 pt or 1"x1" area
-    pdf.drawInlineImage(im, 0, 0, width=72, height=72)
+    pdf.drawInlineImage(eight_by_eight, 0, 0, width=72, height=72)
     pdf.showPage()
     pdf.save()
 
@@ -189,3 +186,38 @@ def test_stack_abuse():
     with pytest.warns(None):
         with pytest.raises(RuntimeError):
             pdfinfo.info._interpret_contents(stream)
+
+
+def test_pages_issue700(monkeypatch, resources):
+    def get_no_pages(*args, **kwargs):
+        return iter([])
+
+    monkeypatch.setattr(PDFPage, 'get_pages', get_no_pages)
+
+    with pytest.raises(InputFileError, match="pdfminer"):
+        pdfinfo.PdfInfo(
+            resources / 'cardinal.pdf',
+            detailed_analysis=True,
+            progbar=False,
+            max_workers=1,
+        )
+
+
+def test_image_scale0(resources, outpdf):
+    with pikepdf.open(resources / 'cmyk.pdf') as cmyk:
+        xobj = cmyk.pages[0].as_form_xobject()
+
+        p = pikepdf.Pdf.new()
+        p.add_blank_page(page_size=(72, 72))
+        objname = p.pages[0].add_resource(
+            p.copy_foreign(xobj), pikepdf.Name.XObject, pikepdf.Name.Im0
+        )
+        print(objname)
+        p.pages[0].Contents = pikepdf.Stream(
+            p, b"q 0 0 0 0 0 0 cm %s Do Q" % bytes(objname)
+        )
+        p.save(outpdf)
+
+    pi = pdfinfo.PdfInfo(outpdf, detailed_analysis=True, progbar=False, max_workers=1)
+    assert not pi.pages[0]._images[0].dpi.is_finite
+    assert pi.pages[0].dpi == Resolution(0, 0)
