@@ -1,53 +1,29 @@
 # © 2019 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 from math import isclose
 
 import pytest
 from PIL import Image
 
-from ocrmypdf._exec import ghostscript
+from ocrmypdf._exec import ghostscript, tesseract
 from ocrmypdf.helpers import Resolution
-from ocrmypdf.leptonica import Pix
 from ocrmypdf.pdfinfo import PdfInfo
 
-# pytest.helpers is dynamic
-# pylint: disable=no-member,redefined-outer-name
-
-check_ocrmypdf = pytest.helpers.check_ocrmypdf
-run_ocrmypdf = pytest.helpers.run_ocrmypdf
-run_ocrmypdf_api = pytest.helpers.run_ocrmypdf_api
-
+from .conftest import check_ocrmypdf, have_unpaper
 
 RENDERERS = ['hocr', 'sandwich']
 
 
 def test_deskew(resources, outdir):
     # Run with deskew
-    deskewed_pdf = check_ocrmypdf(
-        resources / 'skew.pdf',
-        outdir / 'skew.pdf',
-        '-d',
-        '--plugin',
-        'tests/plugins/tesseract_noop.py',
-    )
+    deskewed_pdf = check_ocrmypdf(resources / 'skew.pdf', outdir / 'skew.pdf', '-d')
 
-    # Now render as an image again and use Leptonica to find the skew angle
-    # to confirm that it was deskewed
+    # Now render as an image again...
     deskewed_png = outdir / 'deskewed.png'
 
     ghostscript.rasterize_pdf(
@@ -58,13 +34,18 @@ def test_deskew(resources, outdir):
         pageno=1,
     )
 
-    pix = Pix.open(deskewed_png)
-    skew_angle, _skew_confidence = pix.find_skew()
-
+    # ...and use Tessera to find the skew angle to confirm that it was deskewed
+    skew_angle = tesseract.get_deskew(deskewed_png, [], None, 5.0)
     print(skew_angle)
     assert -0.5 < skew_angle < 0.5, "Deskewing failed"
 
 
+def test_deskew_blank_page(resources, outpdf):
+    # Tesseract doesn't like blank pages - make sure we can get through
+    check_ocrmypdf(resources / 'blank.pdf', outpdf, '--deskew')
+
+
+@pytest.mark.xfail(reason="remove background disabled")
 def test_remove_background(resources, outdir):
     # Ensure the input image does not contain pure white/black
     with Image.open(resources / 'congress.jpg') as im:
@@ -106,7 +87,7 @@ def test_exotic_image(pdf, renderer, output_type, resources, outdir):
     check_ocrmypdf(
         resources / pdf,
         outfile,
-        '-dc' if pytest.helpers.have_unpaper() else '-d',
+        '-dc' if have_unpaper() else '-d',
         '-v',
         '1',
         '--output-type',
