@@ -1,50 +1,35 @@
 # © 2018 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+
+import operator
 from io import BytesIO
+from math import cos, pi, sin
 from os import fspath
 
 import img2pdf
 import pikepdf
 import pytest
-from PIL import Image
+from PIL import Image, ImageChops
+from reportlab.pdfgen.canvas import Canvas
 
-from ocrmypdf import leptonica
-from ocrmypdf._exec import ghostscript, tesseract
-from ocrmypdf.helpers import Resolution
+from ocrmypdf._exec import ghostscript
+from ocrmypdf._plugin_manager import get_plugin_manager
+from ocrmypdf.helpers import IMG2PDF_KWARGS, Resolution
 from ocrmypdf.pdfinfo import PdfInfo
 
-# pytest.helpers is dynamic
-# pylint: disable=no-member
-# pylint: disable=w0612
+from .conftest import check_ocrmypdf, run_ocrmypdf
 
-pytestmark = pytest.mark.skipif(
-    leptonica.get_leptonica_version() < 'leptonica-1.72',
-    reason="Leptonica is too old, correlation doesn't work",
-)
-
-check_ocrmypdf = pytest.helpers.check_ocrmypdf
-run_ocrmypdf = pytest.helpers.run_ocrmypdf
+# pylintx: disable=unused-variable
 
 
 RENDERERS = ['hocr', 'sandwich']
 
 
-def check_monochrome_correlation(
+def compare_images_monochrome(
     outdir, reference_pdf, reference_pageno, test_pdf, test_pageno
 ):
     reference_png = outdir / f'{reference_pdf.name}.ref{reference_pageno:04d}.png'
@@ -66,31 +51,43 @@ def check_monochrome_correlation(
     rasterize(reference_pdf, reference_pageno, reference_png)
     rasterize(test_pdf, test_pageno, test_png)
 
-    pix_ref = leptonica.Pix.open(reference_png)
-    pix_test = leptonica.Pix.open(test_png)
+    with Image.open(reference_png) as reference_im, Image.open(test_png) as test_im:
+        assert reference_im.mode == test_im.mode == '1'
+        difference = ImageChops.logical_xor(reference_im, test_im)
+        assert difference.mode == '1'
 
-    return leptonica.Pix.correlation_binary(pix_ref, pix_test)
+        histogram = difference.histogram()
+        assert (
+            len(histogram) == 256
+        ), "Expected Pillow to convert to grayscale for histogram"
+
+        # All entries other than first and last will be 0
+        count_same = histogram[0]
+        count_different = histogram[-1]
+        total = count_same + count_different
+
+        return count_same / (total)
 
 
-def test_monochrome_correlation(resources, outdir):
-    # Verify leptonica: check that an incorrect rotated image has poor
-    # correlation with reference
-    corr = check_monochrome_correlation(
+def test_monochrome_comparison(resources, outdir):
+    # Self test: check that an incorrect rotated image has poor
+    # comparison with reference
+    cmp = compare_images_monochrome(
         outdir,
         reference_pdf=resources / 'cardinal.pdf',
         reference_pageno=1,  # north facing page
         test_pdf=resources / 'cardinal.pdf',
         test_pageno=3,  # south facing page
     )
-    assert corr < 0.10
-    corr = check_monochrome_correlation(
+    assert cmp < 0.90
+    cmp = compare_images_monochrome(
         outdir,
         reference_pdf=resources / 'cardinal.pdf',
         reference_pageno=2,
         test_pdf=resources / 'cardinal.pdf',
         test_pageno=2,
     )
-    assert corr > 0.90
+    assert cmp > 0.95
 
 
 @pytest.mark.slow
@@ -98,7 +95,7 @@ def test_monochrome_correlation(resources, outdir):
 def test_autorotate(renderer, resources, outdir):
     # cardinal.pdf contains four copies of an image rotated in each cardinal
     # direction - these ones are "burned in" not tagged with /Rotate
-    out = check_ocrmypdf(
+    check_ocrmypdf(
         resources / 'cardinal.pdf',
         outdir / 'out.pdf',
         '-r',
@@ -110,25 +107,25 @@ def test_autorotate(renderer, resources, outdir):
         'tests/plugins/tesseract_cache.py',
     )
     for n in range(1, 4 + 1):
-        correlation = check_monochrome_correlation(
+        cmp = compare_images_monochrome(
             outdir,
             reference_pdf=resources / 'cardinal.pdf',
             reference_pageno=1,
             test_pdf=outdir / 'out.pdf',
             test_pageno=n,
         )
-        assert correlation > 0.80
+        assert cmp > 0.95
 
 
 @pytest.mark.parametrize(
-    'threshold, correlation_test',
+    'threshold, op, comparison_threshold',
     [
-        ('1', 'correlation > 0.80'),  # Low thresh -> always rotate -> high corr
-        ('99', 'correlation < 0.10'),  # High thres -> never rotate -> low corr
+        ('1', operator.ge, 0.95),  # Low thresh -> always rotate -> high score
+        ('99', operator.le, 0.90),  # High thres -> never rotate -> low score
     ],
 )
-def test_autorotate_threshold(threshold, correlation_test, resources, outdir):
-    out = check_ocrmypdf(
+def test_autorotate_threshold(threshold, op, comparison_threshold, resources, outdir):
+    check_ocrmypdf(
         resources / 'cardinal.pdf',
         outdir / 'out.pdf',
         '--rotate-pages-threshold',
@@ -140,14 +137,15 @@ def test_autorotate_threshold(threshold, correlation_test, resources, outdir):
         'tests/plugins/tesseract_cache.py',
     )
 
-    correlation = check_monochrome_correlation(
+    cmp = compare_images_monochrome(  # pylint: disable=unused-variable
         outdir,
         reference_pdf=resources / 'cardinal.pdf',
         reference_pageno=1,
         test_pdf=outdir / 'out.pdf',
         test_pageno=3,
     )
-    assert eval(correlation_test)  # pylint: disable=w0123
+
+    assert op(cmp, comparison_threshold)
 
 
 def test_rotated_skew_timeout(resources, outpdf):
@@ -188,6 +186,7 @@ def test_rotated_skew_timeout(resources, outpdf):
     ), "Expected page rotation to be baked in"
 
 
+@pytest.mark.xfail(reason="tesseract timeout blocks tesseract based deskew")
 def test_rotate_deskew_timeout(resources, outdir):
     check_ocrmypdf(
         resources / 'rotated_skew.pdf',
@@ -202,7 +201,7 @@ def test_rotate_deskew_timeout(resources, outdir):
         'sandwich',
     )
 
-    correlation = check_monochrome_correlation(
+    cmp = compare_images_monochrome(
         outdir,
         reference_pdf=resources / 'ccitt.pdf',
         reference_pageno=1,
@@ -211,7 +210,7 @@ def test_rotate_deskew_timeout(resources, outdir):
     )
 
     # Confirm that the page still got deskewed
-    assert correlation > 0.50
+    assert cmp > 0.95
 
 
 @pytest.mark.slow
@@ -231,6 +230,7 @@ def test_rotate_page_level(image_angle, page_angle, resources, outdir):
             memimg.read(),
             layout_fun=img2pdf.get_fixed_dpi_layout_fun((200, 200)),
             outputstream=mempdf,
+            **IMG2PDF_KWARGS,
         )
         mempdf.seek(0)
         pike = pikepdf.open(mempdf)
@@ -243,26 +243,93 @@ def test_rotate_page_level(image_angle, page_angle, resources, outdir):
     test = make_rotate_test('test', image_angle, page_angle)
     out = test.with_suffix('.out.pdf')
 
-    p, _, err = run_ocrmypdf(
+    p = run_ocrmypdf(
         test,
         out,
         '-O0',
         '--rotate-pages',
         '--rotate-pages-threshold',
         '0.001',
-        universal_newlines=False,
+        text=False,
     )
-    err = err.decode('utf-8', errors='replace')
+    err = p.stderr.decode('utf-8', errors='replace')
     assert p.returncode == 0, err
 
-    assert check_monochrome_correlation(outdir, reference, 1, out, 1) > 0.2
+    assert compare_images_monochrome(outdir, reference, 1, out, 1) > 0.2
 
 
-def test_tesseract_orientation(resources, tmp_path):
-    pix = leptonica.Pix.open(resources / 'crom.png')
-    pix_rotated = pix.rotate_orth(2)  # 180 degrees clockwise
-    pix_rotated.write_implied_format(tmp_path / '000001.png')
+def test_rasterize_rotates(resources, tmp_path):
+    pm = get_plugin_manager([])
 
-    tesseract.get_orientation(  # Test results of this are unreliable
-        tmp_path / '000001.png', engine_mode='3', timeout=10
+    img = tmp_path / 'img90.png'
+    pm.hook.rasterize_pdf_page(
+        input_file=resources / 'graph.pdf',
+        output_file=img,
+        raster_device='pngmono',
+        raster_dpi=Resolution(20, 20),
+        page_dpi=Resolution(20, 20),
+        pageno=1,
+        rotation=90,
+        filter_vector=False,
     )
+    assert Image.open(img).size == (123, 151), "Image not rotated"
+
+    img = tmp_path / 'img180.png'
+    pm.hook.rasterize_pdf_page(
+        input_file=resources / 'graph.pdf',
+        output_file=img,
+        raster_device='pngmono',
+        raster_dpi=Resolution(20, 20),
+        page_dpi=Resolution(20, 20),
+        pageno=1,
+        rotation=180,
+        filter_vector=False,
+    )
+    assert Image.open(img).size == (151, 123), "Image not rotated"
+
+
+def test_simulated_scan(outdir):
+    canvas = Canvas(
+        fspath(outdir / 'fakescan.pdf'),
+        pagesize=(209.8, 297.6),
+    )
+
+    page_vars = [(2, 36, 250), (91, 170, 240), (179, 190, 36), (271, 36, 36)]
+
+    for n, page_var in enumerate(page_vars):
+        text = canvas.beginText()
+        text.setFont('Helvetica', 20)
+
+        angle, x, y = page_var
+        cos_a, sin_a = cos(angle / 180.0 * pi), sin(angle / 180.0 * pi)
+
+        text.setTextTransform(cos_a, -sin_a, sin_a, cos_a, x, y)
+        text.textOut(f'Page {n + 1}')
+        canvas.drawText(text)
+        canvas.showPage()
+    canvas.save()
+
+    check_ocrmypdf(
+        outdir / 'fakescan.pdf',
+        outdir / 'out.pdf',
+        '--force-ocr',
+        '--deskew',
+        '--rotate-pages',
+        '--plugin',
+        'tests/plugins/tesseract_debug_rotate.py',
+    )
+
+    with pikepdf.open(outdir / 'out.pdf') as pdf:
+        assert (
+            pdf.pages[1].MediaBox[2] > pdf.pages[1].MediaBox[3]
+        ), "Wrong orientation: not landscape"
+        assert (
+            pdf.pages[3].MediaBox[2] > pdf.pages[3].MediaBox[3]
+        ), "Wrong orientation: Not landscape"
+
+        assert (
+            pdf.pages[0].MediaBox[2] < pdf.pages[0].MediaBox[3]
+        ), "Wrong orientation: Not portrait"
+        assert (
+            pdf.pages[2].MediaBox[2] < pdf.pages[2].MediaBox[3]
+        ), "Wrong orientation: Not portrait"
