@@ -1,33 +1,28 @@
 # © 2019 James R. Barlow: github.com/jbarlow83
 #
-# This file is part of OCRmyPDF.
-#
-# OCRmyPDF is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# OCRmyPDF is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 import logging
+import os
 from unittest.mock import patch
 
 import pikepdf
 import pytest
 
-import ocrmypdf._validation as vd
+from ocrmypdf import _validation as vd
+from ocrmypdf._concurrent import NullProgressBar, SerialExecutor
+from ocrmypdf._exec.tesseract import TesseractVersion
 from ocrmypdf._plugin_manager import get_plugin_manager
 from ocrmypdf.api import create_options
 from ocrmypdf.cli import get_parser
 from ocrmypdf.exceptions import BadArgsError, MissingDependencyError
 from ocrmypdf.pdfinfo import PdfInfo
 
+from .conftest import run_ocrmypdf_api
+
 
 def make_opts_pm(input_file='a.pdf', output_file='b.pdf', language='eng', **kwargs):
     if language is not None:
@@ -59,29 +54,23 @@ def test_hocr_notlatin_warning(caplog):
 
 
 def test_old_ghostscript(caplog):
-    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.19'), patch(
-        'ocrmypdf._exec.tesseract.has_textonly_pdf', return_value=True
-    ):
+    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.19'):
         vd._check_options(
             *make_opts_pm(language='chi_sim', output_type='pdfa'), {'chi_sim'}
         )
-        assert 'Ghostscript does not work correctly' in caplog.text
+        assert 'does not work correctly' in caplog.text
 
-    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.18'), patch(
-        'ocrmypdf._exec.tesseract.has_textonly_pdf', return_value=True
-    ):
+    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.18'):
         with pytest.raises(MissingDependencyError):
             vd._check_options(*make_opts_pm(output_type='pdfa-3'), set())
 
-    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.24'), patch(
-        'ocrmypdf._exec.tesseract.has_textonly_pdf', return_value=True
-    ):
+    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.24'):
         with pytest.raises(MissingDependencyError):
             vd._check_options(*make_opts_pm(), set())
 
 
 def test_old_tesseract_error():
-    with patch('ocrmypdf._exec.tesseract.has_textonly_pdf', return_value=False):
+    with patch('ocrmypdf._exec.tesseract.version', return_value='4.00.00alpha'):
         with pytest.raises(MissingDependencyError):
             opts = make_opts(pdf_renderer='sandwich', language='eng')
             plugin_manager = get_plugin_manager(opts.plugins)
@@ -100,8 +89,6 @@ def test_mutex_options():
         vd.check_options_ocr_behavior(make_opts(redo_ocr=True, skip_text=True))
     with pytest.raises(BadArgsError):
         vd.check_options_ocr_behavior(make_opts(redo_ocr=True, force_ocr=True))
-    with pytest.raises(BadArgsError):
-        vd.check_options_ocr_behavior(make_opts(pages='1-3', sidecar='file.txt'))
 
 
 def test_optimizing(caplog):
@@ -116,13 +103,19 @@ def test_user_words(caplog):
         opts = make_opts(user_words='foo')
         plugin_manager = get_plugin_manager(opts.plugins)
         vd._check_options(opts, plugin_manager, set())
-        assert '4.0 ignores --user-words' in caplog.text
+        assert (
+            'Tesseract 4.0 (which you have installed) ignores --user-words'
+            in caplog.text
+        )
     caplog.clear()
     with patch('ocrmypdf._exec.tesseract.has_user_words', return_value=True):
         opts = make_opts(user_patterns='foo')
         plugin_manager = get_plugin_manager(opts.plugins)
         vd._check_options(opts, plugin_manager, set())
-        assert '4.0 ignores --user-words' not in caplog.text
+        assert (
+            'Tesseract 4.0 (which you have installed) ignores --user-words'
+            not in caplog.text
+        )
 
 
 def test_pillow_options():
@@ -141,15 +134,15 @@ def test_report_file_size(tmp_path, caplog):
     pdf = pikepdf.new()
     pdf.save(in_)
     pdf.save(out)
-    opts = make_opts()
+    opts = make_opts(output_type='pdf')
     vd.report_output_file_size(opts, in_, out)
     assert caplog.text == ''
     caplog.clear()
 
     waste_of_space = b'Dummy' * 5000
-    pdf.root.Dummy = waste_of_space
+    pdf.Root.Dummy = waste_of_space
     pdf.save(in_)
-    pdf.root.Dummy2 = waste_of_space + waste_of_space
+    pdf.Root.Dummy2 = waste_of_space + waste_of_space
     pdf.save(out)
 
     with patch('ocrmypdf._validation.jbig2enc.available', return_value=True), patch(
@@ -166,7 +159,7 @@ def test_report_file_size(tmp_path, caplog):
         assert 'optional dependency' in caplog.text
     caplog.clear()
 
-    opts = make_opts(in_, out, optimize=0)
+    opts = make_opts(in_, out, optimize=0, output_type='pdf')
     vd.report_output_file_size(opts, in_, out)
     assert 'disabled' in caplog.text
     caplog.clear()
@@ -183,33 +176,44 @@ def test_false_action_store_true():
 def test_no_progress_bar(progress_bar, resources):
     opts = make_opts(progress_bar=progress_bar, input_file=(resources / 'trivial.pdf'))
     plugin_manager = get_plugin_manager(opts.plugins)
-    with patch('ocrmypdf._concurrent.tqdm', autospec=True) as tqdmpatch:
-        vd._check_options(opts, plugin_manager, set())
-        pdfinfo = PdfInfo(opts.input_file, progbar=opts.progress_bar)
-        assert pdfinfo is not None
-        assert tqdmpatch.called
-        _args, kwargs = tqdmpatch.call_args
-        assert kwargs['disable'] != progress_bar
+
+    vd._check_options(opts, plugin_manager, set())
+
+    pbar_disabled = None
+
+    class CheckProgressBar(NullProgressBar):
+        def __init__(self, disable, **kwargs):
+            nonlocal pbar_disabled
+            pbar_disabled = disable
+            super().__init__(disable=disable, **kwargs)
+
+    executor = SerialExecutor(pbar_class=CheckProgressBar)
+    pdfinfo = PdfInfo(opts.input_file, progbar=opts.progress_bar, executor=executor)
+
+    assert pdfinfo is not None
+    assert pbar_disabled is not None and pbar_disabled != progress_bar
 
 
 def test_language_warning(caplog):
     opts = make_opts(language=None)
-    plugin_manager = get_plugin_manager(opts.plugins)
+    _plugin_manager = get_plugin_manager(opts.plugins)
     caplog.set_level(logging.DEBUG)
     with patch(
         'ocrmypdf._validation.locale.getlocale', return_value=('en_US', 'UTF-8')
-    ):
+    ) as mock:
         vd.check_options_languages(opts, {'eng'})
         assert opts.languages == {'eng'}
         assert '' in caplog.text
+        mock.assert_called_once()
 
     opts = make_opts(language=None)
     with patch(
         'ocrmypdf._validation.locale.getlocale', return_value=('fr_FR', 'UTF-8')
-    ):
+    ) as mock:
         vd.check_options_languages(opts, {'eng'})
         assert opts.languages == {'eng'}
         assert 'assuming --language' in caplog.text
+        mock.assert_called_once()
 
 
 def test_version_comparison():
@@ -225,17 +229,41 @@ def test_version_comparison():
         version_checker=lambda: '10.0',
         need_version='8.0.2',
     )
-    vd.check_external_program(
-        program="tesseract",
-        package="tesseract",
-        version_checker=lambda: '4.0.0-beta.1',
-        need_version='4.0.0',
-    )
+    with pytest.raises(MissingDependencyError):
+        vd.check_external_program(
+            program="tesseract",
+            package="tesseract",
+            version_checker=lambda: '4.0.0-beta.1',
+            need_version='4.0.0',
+            version_parser=TesseractVersion,
+        )
     vd.check_external_program(
         program="tesseract",
         package="tesseract",
         version_checker=lambda: 'v5.0.0-alpha.20200201',
         need_version='4.0.0',
+        version_parser=TesseractVersion,
+    )
+    vd.check_external_program(
+        program="tesseract",
+        package="tesseract",
+        version_checker=lambda: '5.0.0-rc1.20211030',
+        need_version='4.0.0',
+        version_parser=TesseractVersion,
+    )
+    vd.check_external_program(
+        program="tesseract",
+        package="tesseract",
+        version_checker=lambda: 'v4.0.0.20181030',  # Some Windows builds use this format
+        need_version='4.0.0',
+        version_parser=TesseractVersion,
+    )
+    vd.check_external_program(
+        program="tesseract",
+        package="tesseract",
+        version_checker=lambda: '4.1.1-rc2-25-g9707',
+        need_version='4.0.0',
+        version_parser=TesseractVersion,
     )
     with pytest.raises(MissingDependencyError):
         vd.check_external_program(
@@ -275,7 +303,17 @@ def test_pagesegmode_warning(caplog):
 
 
 def test_two_languages():
-    with patch('ocrmypdf._exec.tesseract.has_textonly_pdf', return_value=True):
-        vd._check_options(
-            *make_opts_pm(language='fakelang1+fakelang2'), {'fakelang1', 'fakelang2'}
-        )
+    vd._check_options(
+        *make_opts_pm(language='fakelang1+fakelang2'), {'fakelang1', 'fakelang2'}
+    )
+
+
+def test_sidecar_equals_output(resources, no_outpdf):
+    op = no_outpdf
+    with pytest.raises(BadArgsError, match=r'--sidecar'):
+        run_ocrmypdf_api(resources / 'trivial.pdf', op, '--sidecar', op)
+
+
+def test_devnull_sidecar(resources):
+    with pytest.raises(BadArgsError, match=r'--sidecar.*NUL'):
+        run_ocrmypdf_api(resources / 'trivial.pdf', os.devnull, '--sidecar')
